@@ -1,9 +1,8 @@
 import * as Three from 'three';
 import createGrid from './grid-creator';
-import convert from 'convert-units';
 import {disposeObject} from './three-memory-cleaner';
 
-export function parseData(sceneData, editingActions, catalog) {
+export function parseData(sceneData, actions, catalog) {
 
   let planData = {};
 
@@ -23,32 +22,9 @@ export function parseData(sceneData, editingActions, catalog) {
 
   sceneData.layers.forEach(layer => {
 
-    planData.sceneGraph.layers[layer.id] = {
-      lines: {},
-      holes: {},
-      areas: {},
-      items: {},
-      visible: layer.visible,
-      altitude: layer.altitude
-    };
-
-    // Import lines
-    layer.lines.forEach(line => {
-      promises.push(addLine(sceneData, planData, layer, line.id, catalog, editingActions));
-      line.holes.forEach(holeID => {
-        promises.push(addHole(sceneData, planData, layer, holeID, catalog, editingActions));
-      })
-    });
-
-    // Import areas
-    layer.areas.forEach(area => {
-      promises.push(addArea(sceneData, planData, layer, area.id, catalog, editingActions));
-    });
-
-    // Import items
-    layer.items.forEach(item => {
-      promises.push(addItem(sceneData, planData, layer, item.id, catalog, editingActions));
-    });
+    if (layer.id === sceneData.selectedLayer || layer.visible) {
+      promises = promises.concat(createLayerObjects(layer, planData, sceneData, actions, catalog));
+    }
   });
 
   Promise.all(promises).then(value => {
@@ -58,7 +34,41 @@ export function parseData(sceneData, editingActions, catalog) {
   return planData;
 }
 
-export function updateScene(planData, sceneData, oldSceneData, diffArray, editingActions, catalog) {
+function createLayerObjects(layer, planData, sceneData, actions, catalog) {
+
+  let promises = [];
+
+  planData.sceneGraph.layers[layer.id] = {
+    lines: {},
+    holes: {},
+    areas: {},
+    items: {},
+    visible: layer.visible,
+    altitude: layer.altitude
+  };
+
+  // Import lines
+  layer.lines.forEach(line => {
+    promises.push(addLine(sceneData, planData, layer, line.id, catalog, actions.linesActions));
+    line.holes.forEach(holeID => {
+      promises.push(addHole(sceneData, planData, layer, holeID, catalog, actions.holesActions));
+    })
+  });
+
+  // Import areas
+  layer.areas.forEach(area => {
+    promises.push(addArea(sceneData, planData, layer, area.id, catalog, actions.areaActions));
+  });
+
+  // Import items
+  layer.items.forEach(item => {
+    promises.push(addItem(sceneData, planData, layer, item.id, catalog, actions.itemsActions));
+  });
+  
+  return promises;
+}
+
+export function updateScene(planData, sceneData, oldSceneData, diffArray, actions, catalog) {
 
   minimizeChangePropertiesDiffs(diffArray).forEach(diff => {
 
@@ -73,24 +83,59 @@ export function updateScene(planData, sceneData, oldSceneData, diffArray, editin
 
         switch (diff.op) {
           case 'replace':
-            replaceObject(modifiedPath, layer, planData, editingActions, sceneData, oldSceneData, catalog);
+            replaceObject(modifiedPath, layer, planData, actions, sceneData, oldSceneData, catalog);
             break;
           case 'add':
-            addObject(modifiedPath, layer, planData, editingActions, sceneData, oldSceneData, catalog);
+            addObject(modifiedPath, layer, planData, actions, sceneData, oldSceneData, catalog);
             break;
           case 'remove':
-            removeObject(modifiedPath, layer, planData, editingActions, sceneData, oldSceneData, catalog);
+            removeObject(modifiedPath, layer, planData, actions, sceneData, oldSceneData, catalog);
             break;
         }
       }
+    } else if (modifiedPath[1] === 'selectedLayer') {
+      let layerSelectedID = diff.value;
+      // First of all I check if the new selected layer is not visible
+      if (!sceneData.layers.get(layerSelectedID).visible) {
+        // I need to create the objects for this layer
+        let promises = createLayerObjects(sceneData.layers.get(layerSelectedID), planData, sceneData, actions, catalog);
+        Promise.all(promises).then(values => {
+          updateBoundingBox(planData);
+        })
+      }
+
+      // Now I have to ckeck the old selectedLayer
+      let oldLayerSelectedID = oldSceneData.selectedLayer;
+      // First of all I check if the new selected layer is not visible
+      if (!sceneData.layers.get(oldLayerSelectedID).visible) {
+        // I need to remove the objects for this layer
+        let layerGraph = planData.sceneGraph.layers[oldLayerSelectedID];
+
+        for (let lineID in layerGraph.lines) {
+          removeLine(planData, sceneData.layers.get(oldLayerSelectedID), lineID);
+        }
+
+        for (let areaID in layerGraph.areas) {
+          removeArea(planData, sceneData.layers.get(oldLayerSelectedID), areaID);
+        }
+
+        for (let itemID in layerGraph.items) {
+          removeItem(planData, sceneData.layers.get(oldLayerSelectedID), itemID);
+        }
+
+        for (let holeID in layerGraph.holes) {
+          removeHole(planData, sceneData.layers.get(oldLayerSelectedID), holeID);
+        }
+      }
     }
   });
   return planData;
 }
 
-function replaceObject(modifiedPath, layer, planData, editingActions, sceneData, oldSceneData, catalog) {
+function replaceObject(modifiedPath, layer, planData, actions, sceneData, oldSceneData, catalog) {
 
   let promises = [];
+
   switch (modifiedPath[3]) {
     case "layer":
       break;
@@ -102,49 +147,55 @@ function replaceObject(modifiedPath, layer, planData, editingActions, sceneData,
       if (modifiedPath[5] === 'selected') {
         // I remove only the hole without removing the wall
         removeHole(planData, layer, newHoleData.id);
-        promises.push(addHole(sceneData, planData, layer, newHoleData.id, catalog, editingActions));
+        promises.push(addHole(sceneData, planData, layer, newHoleData.id, catalog, actions.holesActions));
       } else {
         layer.lines.get(lineID).holes.forEach(holeID => {
           removeHole(planData, layer, holeID);
         });
         removeLine(planData, layer, lineID);
-        promises.push(addLine(sceneData, planData, layer, lineID, catalog, editingActions));
+        promises.push(addLine(sceneData, planData, layer, lineID, catalog, actions.linesActions));
         layer.lines.get(lineID).holes.forEach(holeID => {
-          promises.push(addHole(sceneData, planData, layer, holeID, catalog, editingActions));
+          promises.push(addHole(sceneData, planData, layer, holeID, catalog, actions.holesActions));
         });
       }
       break;
     case "lines":
       removeLine(planData, layer, modifiedPath[4]);
-      promises.push(addLine(sceneData, planData, layer, modifiedPath[4], catalog, editingActions));
+      promises.push(addLine(sceneData, planData, layer, modifiedPath[4], catalog, actions.linesActions));
       break;
     case "areas":
       removeArea(planData, layer, modifiedPath[4]);
-      promises.push(addArea(sceneData, planData, layer, modifiedPath[4], catalog, editingActions));
+      promises.push(addArea(sceneData, planData, layer, modifiedPath[4], catalog, actions.areaActions));
       break;
     case "items":
       removeItem(planData, layer, modifiedPath[4]);
-      promises.push(addItem(sceneData, planData, layer, modifiedPath[4], catalog, editingActions));
+      promises.push(addItem(sceneData, planData, layer, modifiedPath[4], catalog, actions.itemsActions));
       break;
 
     case "visible":
-      let layerGraph = planData.sceneGraph.layers[layer.id];
-      layerGraph.visible = layer.visible;
-      for (let lineID in layerGraph.lines) {
-        layerGraph.lines[lineID].visible = layer.visible;
+      if (!layer.visible) {
+        let layerGraph = planData.sceneGraph.layers[layer.id];
+
+        for (let lineID in layerGraph.lines) {
+          removeLine(planData, layer, lineID);
+        }
+
+        for (let areaID in layerGraph.areas) {
+          removeArea(planData, layer, areaID);
+        }
+
+        for (let itemID in layerGraph.items) {
+          removeItem(planData, layer, itemID);
+        }
+
+        for (let holeID in layerGraph.holes) {
+          removeHole(planData, layer, holeID);
+        }
+
+      } else {
+        promises = promises.concat(createLayerObjects(layer, planData, sceneData, actions, catalog))
       }
 
-      for (let areaID in layerGraph.areas) {
-        layerGraph.areas[areaID].visible = layer.visible;
-      }
-
-      for (let itemID in layerGraph.items) {
-        layerGraph.items[itemID].visible = layer.visible;
-      }
-
-      for (let holeID in layerGraph.holes) {
-        layerGraph.holes[holeID].visible = layer.visible;
-      }
       break;
   }
   Promise.all(promises).then(values => {
@@ -152,7 +203,7 @@ function replaceObject(modifiedPath, layer, planData, editingActions, sceneData,
   })
 }
 
-function removeObject(modifiedPath, layer, planData, editingActions, sceneData, oldSceneData, catalog) {
+function removeObject(modifiedPath, layer, planData, actions, sceneData, oldSceneData, catalog) {
 
   let promises = [];
   switch (modifiedPath[3]) {
@@ -166,9 +217,9 @@ function removeObject(modifiedPath, layer, planData, editingActions, sceneData,
       removeLine(planData, layer, lineID);
       if (modifiedPath.length > 5) {
         // I removed an hole, so I should add the new line
-        promises.push(addLine(sceneData, planData, layer, lineID, catalog, editingActions));
+        promises.push(addLine(sceneData, planData, layer, lineID, catalog, actions.linesActions));
         layer.lines.get(lineID).holes.forEach(holeID => {
-          promises.push(addHole(sceneData, planData, layer, holeID, catalog, editingActions));
+          promises.push(addHole(sceneData, planData, layer, holeID, catalog, actions.holesActions));
         });
       }
       break;
@@ -227,26 +278,26 @@ function removeItem(planData, layer, itemID) {
   updateBoundingBox(planData);
 }
 
-function addObject(modifiedPath, layer, planData, editingActions, sceneData, oldSceneData, catalog) {
+function addObject(modifiedPath, layer, planData, actions, sceneData, oldSceneData, catalog) {
 
   let promises = [];
   switch (modifiedPath[3]) {
     case "lines":
       if (modifiedPath.length === 5) {
         // I have to add a line
-        promises.push(addLine(sceneData, planData, layer, modifiedPath[4], catalog, editingActions));
+        promises.push(addLine(sceneData, planData, layer, modifiedPath[4], catalog, actions.linesActions));
       }
       break;
     case "areas":
       if (modifiedPath.length === 5) {
         // I have to add an area
-        promises.push(addArea(sceneData, planData, layer, modifiedPath[4], catalog, editingActions));
+        promises.push(addArea(sceneData, planData, layer, modifiedPath[4], catalog, actions.areaActions));
       }
       break;
     case "items":
       if (modifiedPath.length === 5) {
         // I have to add an area
-        promises.push(addItem(sceneData, planData, layer, modifiedPath[4], catalog, editingActions));
+        promises.push(addItem(sceneData, planData, layer, modifiedPath[4], catalog, actions.itemsActions));
       }
       break;
   }
@@ -256,7 +307,7 @@ function addObject(modifiedPath, layer, planData, editingActions, sceneData, old
   })
 }
 
-function addHole(sceneData, planData, layer, holeID, catalog, editingActions) {
+function addHole(sceneData, planData, layer, holeID, catalog, holesActions) {
   let holeData = layer.holes.get(holeID);
 
   // Create the hole object
@@ -295,19 +346,17 @@ function addHole(sceneData, planData, layer, holeID, catalog, editingActions) {
     pivot.position.y = holeAltitude + holeHeight / 2 - center[1];
     pivot.position.z = -vertex0.y - distance * holeData.offset * Math.sin(alpha) - center[2] * Math.cos(alpha);
 
-    pivot.visible = layer.visible;
-
     planData.plan.add(pivot);
     planData.sceneGraph.layers[layer.id].holes[holeData.id] = pivot;
 
     applyInteract(pivot, () => {
-      return editingActions.selectHole(layer.id, holeData.id)
+      return holesActions.selectHole(layer.id, holeData.id)
     });
 
   });
 }
 
-function addLine(sceneData, planData, layer, lineID, catalog, editingActions) {
+function addLine(sceneData, planData, layer, lineID, catalog, linesActions) {
   let line = layer.lines.get(lineID);
 
   // First of all I need to find the vertices of this line
@@ -329,22 +378,20 @@ function addLine(sceneData, planData, layer, lineID, catalog, editingActions) {
     pivot.position.y = layer.altitude;
     pivot.position.z = -vertex0.y;
 
-    pivot.visible = layer.visible;
-
     planData.plan.add(pivot);
     planData.sceneGraph.layers[layer.id].lines[lineID] = pivot;
 
     applyInteract(pivot, () => {
-      return editingActions.selectLine(layer.id, line.id);
+      return linesActions.selectLine(layer.id, line.id);
     });
 
   });
 }
 
-function addArea(sceneData, planData, layer, areaID, catalog, editingActions) {
+function addArea(sceneData, planData, layer, areaID, catalog, areaActions) {
   let area = layer.areas.get(areaID);
   let interactFunction = () => {
-    editingActions.selectArea(layer.id, area.id);
+    areaActions.selectArea(layer.id, area.id);
   };
 
   return catalog.getElement(area.type).render3D(area, layer, sceneData).then(area3D => {
@@ -353,14 +400,13 @@ function addArea(sceneData, planData, layer, areaID, catalog, editingActions) {
     pivot.position.y = layer.altitude;
     planData.plan.add(pivot);
     planData.sceneGraph.layers[layer.id].areas[area.id] = pivot;
-    pivot.visible = layer.visible;
 
     applyInteract(pivot, interactFunction);
 
   });
 }
 
-function addItem(sceneData, planData, layer, itemID, catalog, editingActions) {
+function addItem(sceneData, planData, layer, itemID, catalog, itemsActions) {
   let item = layer.items.get(itemID);
 
   return catalog.getElement(item.type).render3D(item, layer, sceneData).then(item3D => {
@@ -372,10 +418,8 @@ function addItem(sceneData, planData, layer, itemID, catalog, editingActions) {
     pivot.position.x = item.x;
     pivot.position.z = -item.y;
 
-    pivot.visible = layer.visible;
-
     applyInteract(item3D, () => {
-        editingActions.selectItem(layer.id, item.id);
+        itemsActions.selectItem(layer.id, item.id);
       }
     );
 
